@@ -30,8 +30,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x041100
-sip_version_str = "4.17"
+sip_version = 0x041306
+sip_version_str = "4.19.6"
 py_version = sys.hexversion >> 8
 py_platform = sys.platform
 plat_py_site_dir = None
@@ -46,6 +46,7 @@ sip_bin_dir = ''
 sip_inc_dir = ''
 sip_module_dir = ''
 sip_sip_dir = ''
+pyi_dir = ''
 sysroot = ''
 src_dir = os.path.dirname(os.path.abspath(__file__))
 sip_module_base = None
@@ -183,6 +184,9 @@ def inform_user():
 
     siputils.inform("The %s module will be installed in %s." % (sip_module_base, sip_module_dir))
 
+    if opts.pyi:
+        siputils.inform("The sip.pyi stub file will be installed in %s." % pyi_dir)
+
     if opts.static:
         siputils.inform("The %s module will be built as a static library." % sip_module_base)
 
@@ -351,7 +355,7 @@ def create_makefiles(macros):
         pro.write("TEMPLATE = app\n")
         pro.write("TARGET = sip\n")
         pro.write("CONFIG -= qt app_bundle\n")
-        pro.write("CONFIG += warn_off exceptions_off console %s\n" % (
+        pro.write("CONFIG += warn_on exceptions_off console %s\n" % (
                 ("debug" if opts.debug else "release")))
 
         pro.write("\n")
@@ -380,7 +384,7 @@ def create_makefiles(macros):
             dir="sipgen",
             install_dir=os.path.dirname(cfg.sip_bin),
             console=1,
-            warnings=0,
+            warnings=1,
             universal=opts.universal,
             arch=opts.arch,
             deployment_target=opts.deployment_target
@@ -395,7 +399,7 @@ def create_makefiles(macros):
         pro.write("TARGET = %s\n" % sip_module_base)
         pro.write("CONFIG -= qt\n")
         pro.write("CONFIG += warn_on exceptions_off %s %s\n" % (
-                ("staticlib" if opts.static else "plugin"),
+                ("staticlib" if opts.static else "plugin plugin_bundle"),
                 ("debug" if opts.debug else "release")))
 
         pro.write("\n")
@@ -415,30 +419,39 @@ def create_makefiles(macros):
             pro.write("""
 win32 {
     PY_MODULE = %s%s.pyd
-    target.files = %s%s.pyd
-    LIBS += %s -lpython%d.%d
-    QMAKE_POST_LINK = $(COPY_FILE) $(DESTDIR_TARGET) $$PY_MODULE
+    PY_MODULE_SRC = $(DESTDIR_TARGET)
+
+    LIBS += %s
 } else {
     PY_MODULE = %s.so
-    target.files = %s.so
-    QMAKE_POST_LINK = $(COPY_FILE) $(TARGET) $$PY_MODULE
+
+    macx {
+        PY_MODULE_SRC = $(TARGET).plugin/Contents/MacOS/$(TARGET)
+
+        QMAKE_LFLAGS += "-undefined dynamic_lookup"
+    } else {
+        PY_MODULE_SRC = $(TARGET)
+    }
 }
 
-macx {
-    QMAKE_LFLAGS += "-undefined dynamic_lookup"
-    QMAKE_LFLAGS += "-install_name $$absolute_path($$PY_MODULE, $$target.path)"
-}
+QMAKE_POST_LINK = $(COPY_FILE) $(DESTDIR_TARGET) $$PY_MODULE
+
+target.CONFIG = no_check_exist
+target.files = $$PY_MODULE
 """ % (sip_module_base, debug_suffix,
-       sip_module_base, debug_suffix,
-       link_lib_dir, (py_version >> 16), ((py_version >> 8) & 0xff),
-       sip_module_base,
+       link_lib_dir,
        sip_module_base))
 
         pro.write("\n")
-        pro.write("target.CONFIG = no_check_exist\n")
         pro.write("target.path = %s\n" % cfg.sip_mod_dir)
         pro.write("INSTALLS += target\n")
 
+        if opts.pyi:
+            pro.write("\n")
+            pro.write("sip_pyi.files = sip.pyi\n")
+            pro.write("sip_pyi.path = %s\n" % pyi_dir)
+            pro.write("INSTALLS += sip_pyi\n")
+
         pro.write("\n")
         pro.write("sip_h.files = sip.h\n")
         pro.write("sip_h.path = %s\n" % cfg.sip_inc_dir)
@@ -459,15 +472,21 @@ macx {
 
         build_dir = os.getcwd()
 
+        installs=[
+                ([os.path.join(build_dir, "siplib", "sip.h")],
+                        cfg.sip_inc_dir)]
+
+        if opts.pyi:
+            installs.append(([os.path.join(src_dir, 'sip.pyi')], pyi_dir))
+
         makefile = sipconfig.ModuleMakefile(
             configuration=cfg,
             build_file=os.path.join(build_dir, "siplib", "siplib.sbf"),
             dir="siplib",
             install_dir=cfg.sip_mod_dir,
-            installs=([os.path.join(build_dir, "siplib", "sip.h")],
-                    cfg.sip_inc_dir),
+            installs=installs,
             console=1,
-            warnings=0,
+            warnings=1,
             static=opts.static,
             debug=opts.debug,
             universal=opts.universal,
@@ -576,7 +595,7 @@ def update_from_configuration_file(config_file):
     config['py_minor'] = version[1]
     config['sysroot'] = sysroot
 
-    # Override the relevent values.
+    # Override the relevant values.
     global py_platform, plat_py_conf_inc_dir, plat_py_inc_dir, plat_py_lib_dir
     global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir
 
@@ -713,6 +732,13 @@ def create_optparser(sdk_dir):
             metavar="DIR", dest="sipsipdir", callback=store_abspath,
             help="where .sip files are normally installed [default: "
                     "%s]" % plat_sip_dir)
+    g.add_option("--no-stubs", "--no-pyi", action="store_false", default=True,
+            dest="pyi",
+            help="do not install the sip.pyi stub file")
+    g.add_option("--stubsdir", "--pyidir", action="callback", type="string",
+            metavar="DIR", dest="pyidir", callback=store_abspath,
+            help="where the sip.pyi stub file will be installed [default: "
+                    "%s]" % plat_py_site_dir)
     p.add_option_group(g)
 
     return p
@@ -804,7 +830,7 @@ def main(argv):
     # Apply the overrides from any configuration file.
     global plat_bin_dir, plat_py_conf_inc_dir, plat_py_inc_dir
     global plat_py_lib_dir, plat_py_site_dir, plat_sip_dir
-    global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir
+    global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir, pyi_dir
 
     # Set defaults.
     sip_bin_dir = plat_bin_dir
@@ -849,6 +875,11 @@ def main(argv):
     if opts.sipsipdir is not None:
         sip_sip_dir = opts.sipsipdir
 
+    if opts.pyidir is not None:
+        pyi_dir = opts.pyidir
+    else:
+        pyi_dir = sip_module_dir
+
     # Get the platform specific macros for building.
     macros = siputils.parse_build_macros(
             os.path.join(src_dir, "specs", build_platform), build_macro_names,
